@@ -4,7 +4,7 @@ const Clutter = imports.gi.Clutter;;
 const GLib = imports.gi.GLib;
 const Shell = imports.gi.Shell;
 const St = imports.gi.St;
-const Gettext_gtk20 = imports.gettext.domain('gtk20');
+const Gettext_gtk30 = imports.gettext.domain('gtk30');
 
 const Tweener = imports.ui.tweener;
 
@@ -65,7 +65,7 @@ function init() {
     String.prototype.format = Format.format;
 
     // Set the default direction for St widgets (this needs to be done before any use of St)
-    if (Gettext_gtk20.gettext('default:LTR') == 'default:RTL') {
+    if (Gettext_gtk30.gettext('default:LTR') == 'default:RTL') {
         St.Widget.set_default_direction(St.TextDirection.RTL);
     }
 
@@ -83,10 +83,11 @@ function init() {
         return St.describe_actor(this);
     };
 
+    if (window.global === undefined) // test environment
+        return;
+
     _blockMethod('Clutter.Event.get_state', 'Shell.get_event_state',
                  'gjs\'s handling of Clutter.ModifierType is broken. See bug 597292.');
-    _blockMethod('Gdk.Display.get_device_state', 'global.get_pointer',
-                 'gjs\'s handling of Gdk.ModifierType is broken. See bug 597292.');
     _blockMethod('Gdk.Window.get_device_position', 'global.get_pointer',
                  'gjs\'s handling of Gdk.ModifierType is broken. See bug 597292.');
 
