@@ -16,7 +16,10 @@ const Tweener = imports.ui.tweener;
 const Main = imports.ui.main;
 const BoxPointer = imports.ui.boxpointer;
 const Params = imports.misc.params;
-const Utils = imports.misc.utils;
+const Util = imports.misc.util;
+
+const Gettext = imports.gettext.domain('gnome-shell');
+const _ = Gettext.gettext;
 
 const ANIMATION_TIME = 0.2;
 const NOTIFICATION_TIMEOUT = 4;
@@ -38,6 +41,28 @@ const State = {
     HIDING:  3
 };
 
+// These reasons are useful when we destroy the notifications received through
+// the notification daemon. We use EXPIRED for transient notifications that the
+// user did not interact with, DISMISSED for all other notifications that were
+// destroyed as a result of a user action, and SOURCE_CLOSED for the notifications
+// that were requested to be destroyed by the associated source.
+const NotificationDestroyedReason = {
+    EXPIRED: 1,
+    DISMISSED: 2,
+    SOURCE_CLOSED: 3
+};
+
+// Message tray has its custom Urgency enumeration. LOW, NORMAL and CRITICAL
+// urgency values map to the corresponding values for the notifications received
+// through the notification daemon. HIGH urgency value is used for chats received
+// through the Telepathy client.
+const Urgency = {
+    LOW: 0,
+    NORMAL: 1,
+    HIGH: 2,
+    CRITICAL: 3
+}
+
 function _fixMarkup(text, allowMarkup) {
     if (allowMarkup) {
         // Support &amp;, &quot;, &apos;, &lt; and &gt;, escape all other
@@ -92,8 +117,7 @@ URLHighlighter.prototype = {
                     return true;
                 } catch (e) {
                     // TODO: remove this after gnome 3 release
-                    let p = new Shell.Process({ 'args' : ['gvfs-open', url] });
-                    p.run();
+                    Util.spawn(['gvfs-open', url]);
                     return true;
                 }
             }
@@ -124,13 +148,13 @@ URLHighlighter.prototype = {
 
         this.actor.clutter_text.set_markup(text);
         /* clutter_text.text contain text without markup */
-        this._urls = Utils.findUrls(this.actor.clutter_text.text);
+        this._urls = Util.findUrls(this.actor.clutter_text.text);
         this._highlightUrls();
     },
 
     _highlightUrls: function() {
         // text here contain markup
-        let urls = Utils.findUrls(this._text);
+        let urls = Util.findUrls(this._text);
         let markup = '';
         let pos = 0;
         for (let i = 0; i < urls.length; i++) {
@@ -225,11 +249,12 @@ function Notification(source, title, banner, params) {
 Notification.prototype = {
     _init: function(source, title, banner, params) {
         this.source = source;
-        this.urgent = false;
+        this.urgency = Urgency.NORMAL;
         this.resident = false;
         // 'transient' is a reserved keyword in JS, so we have to use an alternate variable name
         this.isTransient = false;
         this.expanded = false;
+        this._destroyed = false;
         this._useActionIcons = false;
         this._customContent = false;
         this._bannerBodyText = null;
@@ -250,7 +275,11 @@ Notification.prototype = {
         this._capturedEventId = 0;
         this._keyPressId = 0;
 
-        source.connect('destroy', Lang.bind(this, this.destroy));
+        source.connect('destroy', Lang.bind(this,
+            // Avoid passing 'source' as an argument to this.destroy()
+            function () {
+                this.destroy();
+            }));
 
         this.actor = new St.Table({ name: 'notification',
                                     reactive: true });
@@ -373,7 +402,7 @@ Notification.prototype = {
             this._scrollArea = new St.ScrollView({ name: 'notification-scrollview',
                                                    vscrollbar_policy: Gtk.PolicyType.AUTOMATIC,
                                                    hscrollbar_policy: Gtk.PolicyType.NEVER,
-                                                   vshadows: true });
+                                                   vfade: true });
             this.actor.add(this._scrollArea, { row: 1,
                                                col: 1 });
             this._contentArea = new St.BoxLayout({ name: 'notification-body',
@@ -490,8 +519,8 @@ Notification.prototype = {
         this._updated();
     },
 
-    setUrgent: function(urgent) {
-        this.urgent = urgent;
+    setUrgency: function(urgency) {
+        this.urgency = urgency;
     },
 
     setResident: function(resident) {
@@ -693,15 +722,6 @@ Notification.prototype = {
             this.destroy();
     },
 
-    _onKeyPress: function(actor, event) {
-        let symbol = event.get_key_symbol();
-        if (symbol == Clutter.Escape) {
-            Main.messageTray.escapeTray();
-            return true;
-        }
-        return false;
-    },
-
     ungrabFocus: function() {
         if (!this._hasFocus)
             return;
@@ -751,8 +771,13 @@ Notification.prototype = {
         }
     },
 
-    destroy: function() {
-        this.emit('destroy');
+    destroy: function(reason) {
+        if (this._destroyed)
+            return;
+        this._destroyed = true;
+        if (!reason)
+            reason = NotificationDestroyedReason.DISMISSED;
+        this.emit('destroy', reason);
     }
 };
 Signals.addSignalMethods(Notification.prototype);
@@ -905,7 +930,7 @@ MessageTray.prototype = {
         this._notification = null;
         this._notificationClickedId = 0;
 
-        this._summaryBin = new St.Bin({ anchor_gravity: Clutter.Gravity.NORTH_EAST });
+        this._summaryBin = new St.Bin({ x_align: St.Align.END });
         this.actor.add_actor(this._summaryBin);
         this._summary = new St.BoxLayout({ name: 'summary-mode',
                                            reactive: true,
@@ -997,9 +1022,8 @@ MessageTray.prototype = {
         this.actor.width = primary.width;
         this._notificationBin.x = 0;
         this._notificationBin.width = primary.width;
-
-        // These work because of their anchor_gravity
-        this._summaryBin.x = primary.width;
+        this._summaryBin.x = 0;
+        this._summaryBin.width = primary.width;
     },
 
     contains: function(source) {
@@ -1166,13 +1190,11 @@ MessageTray.prototype = {
         } else if (this._notificationQueue.indexOf(notification) < 0) {
             notification.connect('destroy',
                                  Lang.bind(this, this.removeNotification));
-
-            if (notification.urgent)
-                this._notificationQueue.unshift(notification);
-            else
-                this._notificationQueue.push(notification);
+            this._notificationQueue.push(notification);
+            this._notificationQueue.sort(function(notification1, notification2) {
+                return (notification2.urgency - notification1.urgency);
+            });
         }
-
         this._updateState();
     },
 
@@ -1371,7 +1393,7 @@ MessageTray.prototype = {
         let notificationsPending = this._notificationQueue.length > 0;
         let notificationPinned = this._pointerInTray && !this._pointerInSummary && !this._notificationRemoved;
         let notificationExpanded = this._notificationBin.y < 0;
-        let notificationExpired = (this._notificationTimeoutId == 0 && !this._pointerInTray && !this._locked) || this._notificationRemoved;
+        let notificationExpired = (this._notificationTimeoutId == 0 && !(this._notification && this._notification.urgency == Urgency.CRITICAL) && !this._pointerInTray && !this._locked) || this._notificationRemoved;
 
         if (this._notificationState == State.HIDDEN) {
             if (notificationsPending)
@@ -1499,27 +1521,45 @@ MessageTray.prototype = {
 
     _updateShowingNotification: function() {
         Tweener.removeTweens(this._notificationBin);
-        this._tween(this._notificationBin, '_notificationState', State.SHOWN,
-                    { y: 0,
-                      opacity: 255,
-                      time: ANIMATION_TIME,
-                      transition: 'easeOutQuad',
-                      onComplete: this._showNotificationCompleted,
-                      onCompleteScope: this
-                    });
 
-        // We auto-expand urgent notifications.
-        // We call _expandNotification() again on the notifications that
-        // are expanded in case they were in the process of hiding and need
-        // to re-expand.
-        if (this._notification.urgent || this._notification.expanded)
-            // This will overwrite the y tween, but leave the opacity
-            // tween, and so the onComplete will remain as well.
+        // We auto-expand notifications with CRITICAL urgency.
+        // We use Tweener.removeTweens() to remove a tween that was hiding the notification we are
+        // updating, in case that notification was in the process of being hidden. However,
+        // Tweener.removeTweens() would also remove a tween that was updating the position of the
+        // notification we are updating, in case that notification was already expanded and its height
+        // changed. Therefore we need to call this._expandNotification() for expanded notifications
+        // to make sure their position is updated.
+        if (this._notification.urgency == Urgency.CRITICAL || this._notification.expanded)
             this._expandNotification(true);
+
+        // We tween all notifications to full opacity. This ensures that both new notifications and
+        // notifications that might have been in the process of hiding get full opacity.
+        //
+        // We tween any notification showing in the banner mode to banner height (this._notificationBin.y = 0).
+        // This ensures that both new notifications and notifications in the banner mode that might
+        // have been in the process of hiding are shown with the banner height.
+        //
+        // We use this._showNotificationCompleted() onComplete callback to extend the time the updated
+        // notification is being shown.
+        //
+        // We don't set the y parameter for the tween for expanded notifications because
+        // this._expandNotification() will result in getting this._notificationBin.y set to the appropriate
+        // fully expanded value.
+        let tweenParams = { opacity: 255,
+                            time: ANIMATION_TIME,
+                            transition: 'easeOutQuad',
+                            onComplete: this._showNotificationCompleted,
+                            onCompleteScope: this
+                          };
+        if (!this._notification.expanded)
+            tweenParams.y = 0;
+
+        this._tween(this._notificationBin, '_notificationState', State.SHOWN, tweenParams);
    },
 
     _showNotificationCompleted: function() {
-        this._updateNotificationTimeout(NOTIFICATION_TIMEOUT * 1000);
+        if (this._notification.urgency != Urgency.CRITICAL)
+            this._updateNotificationTimeout(NOTIFICATION_TIMEOUT * 1000);
     },
 
     _updateNotificationTimeout: function(timeout) {
@@ -1577,7 +1617,7 @@ MessageTray.prototype = {
         let notification = this._notification;
         this._notification = null;
         if (notification.isTransient)
-            notification.destroy();
+            notification.destroy(NotificationDestroyedReason.EXPIRED);
     },
 
     _expandNotification: function(autoExpanding) {
@@ -1604,7 +1644,7 @@ MessageTray.prototype = {
    },
 
     // We use this function to grab focus when the user moves the pointer
-    // to an urgent notification that was already auto-expanded.
+    // to a notification with CRITICAL urgency that was already auto-expanded.
     _ensureNotificationFocused: function() {
         this._notification.grabFocus(false);
     },
@@ -1727,10 +1767,34 @@ MessageTray.prototype = {
         let summaryNotification = this._summaryNotification;
         this._summaryNotification = null;
         if (summaryNotification.isTransient && !this._reNotifyWithSummaryNotificationAfterHide)
-            summaryNotification.destroy();
+            summaryNotification.destroy(NotificationDestroyedReason.EXPIRED);
         if (this._reNotifyWithSummaryNotificationAfterHide) {
             this._onNotify(summaryNotification.source, summaryNotification);
             this._reNotifyWithSummaryNotificationAfterHide = false;
         }
     }
 };
+
+function SystemNotificationSource() {
+    this._init();
+}
+
+SystemNotificationSource.prototype = {
+    __proto__:  Source.prototype,
+
+    _init: function() {
+        Source.prototype._init.call(this, _("System Information"));
+
+        this._setSummaryIcon(this.createNotificationIcon());
+    },
+
+    createNotificationIcon: function() {
+        return new St.Icon({ icon_name: 'dialog-information',
+                             icon_type: St.IconType.SYMBOLIC,
+                             icon_size: this.ICON_SIZE });
+    },
+
+    _notificationClicked: function() {
+        this.destroy();
+    }
+};
