@@ -16,9 +16,9 @@
 #include "shell-window-tracker-private.h"
 #include "shell-app-system.h"
 #include "shell-app-private.h"
-#include "st/st-texture-cache.h"
 #include "shell-global.h"
 #include "shell-marshal.h"
+#include "st.h"
 
 #include "display.h"
 #include "window.h"
@@ -658,25 +658,7 @@ on_startup_sequence_changed (MetaScreen            *screen,
 
   app = shell_startup_sequence_get_app ((ShellStartupSequence*)sequence);
   if (app)
-    {
-      gboolean starting = !sn_startup_sequence_get_completed (sequence);
-
-      /* The Shell design calls for on application launch, the app title
-       * appears at top, and no X window is focused.  So when we get
-       * a startup-notification for this app, transition it to STARTING
-       * if it's currently stopped, set it as our application focus,
-       * but focus the no_focus window.
-       */
-      if (starting && shell_app_get_state (app) == SHELL_APP_STATE_STOPPED)
-        {
-          MetaScreen *screen = shell_global_get_screen (shell_global_get ());
-          MetaDisplay *display = meta_screen_get_display (screen);
-
-          _shell_app_set_starting (app, starting);
-          meta_display_focus_the_no_focus_window (display, screen,
-                                                  sn_startup_sequence_get_timestamp (sequence));
-        }
-    }
+    _shell_app_handle_startup_sequence (app, sequence);
 
   g_signal_emit (G_OBJECT (self), signals[STARTUP_SEQUENCE_CHANGED], 0, sequence);
 }
@@ -843,12 +825,9 @@ on_focus_window_changed (MetaDisplay        *display,
                          GParamSpec         *spec,
                          ShellWindowTracker *tracker)
 {
-  MetaScreen *screen;
   MetaWindow *new_focus_win;
   ShellApp *new_focus_app;
 
-  screen = shell_global_get_screen (shell_global_get ());
-
   new_focus_win = meta_display_get_focus_window (display);
   new_focus_app = new_focus_win ? g_hash_table_lookup (tracker->window_to_app, new_focus_win) : NULL;
 
@@ -959,7 +938,7 @@ shell_startup_sequence_create_icon (ShellStartupSequence *sequence, guint size)
 
   themed = g_themed_icon_new (icon_name);
   texture = st_texture_cache_load_gicon (st_texture_cache_get_default (),
-                                         themed, size);
+                                         NULL, themed, size);
   g_object_unref (G_OBJECT (themed));
   return texture;
 }
