@@ -49,6 +49,8 @@ G_DEFINE_TYPE (StIcon, st_icon, ST_TYPE_WIDGET)
 struct _StIconPrivate
 {
   ClutterActor *icon_texture;
+  ClutterActor *pending_texture;
+  guint         opacity_handler_id;
 
   GIcon        *gicon;
   gchar        *icon_name;
@@ -60,6 +62,7 @@ struct _StIconPrivate
   CoglHandle    shadow_material;
   float         shadow_width;
   float         shadow_height;
+  StShadow     *shadow_spec;
 };
 
 static void st_icon_update               (StIcon *icon);
@@ -143,6 +146,13 @@ st_icon_dispose (GObject *gobject)
       priv->icon_texture = NULL;
     }
 
+  if (priv->pending_texture)
+    {
+      clutter_actor_destroy (priv->pending_texture);
+      g_object_unref (priv->pending_texture);
+      priv->pending_texture = NULL;
+    }
+
   if (priv->gicon)
     {
       g_object_unref (priv->gicon);
@@ -155,6 +165,12 @@ st_icon_dispose (GObject *gobject)
       priv->shadow_material = COGL_INVALID_HANDLE;
     }
 
+  if (priv->shadow_spec)
+    {
+      st_shadow_unref (priv->shadow_spec);
+      priv->shadow_spec = NULL;
+    }
+
   G_OBJECT_CLASS (st_icon_parent_class)->dispose (gobject);
 }
 
@@ -240,8 +256,6 @@ st_icon_paint (ClutterActor *actor)
     {
       if (priv->shadow_material)
         {
-          StThemeNode *node = st_widget_get_theme_node (ST_WIDGET (actor));
-          StShadow *shadow_spec = st_theme_node_get_shadow (node);
           ClutterActorBox allocation;
           float width, height;
 
@@ -253,7 +267,7 @@ st_icon_paint (ClutterActor *actor)
           allocation.x2 = allocation.x1 + priv->shadow_width;
           allocation.y2 = allocation.y1 + priv->shadow_height;
 
-          _st_paint_shadow_with_opacity (shadow_spec,
+          _st_paint_shadow_with_opacity (priv->shadow_spec,
                                          priv->shadow_material,
                                          &allocation,
                                          clutter_actor_get_paint_opacity (priv->icon_texture));
@@ -263,28 +277,6 @@ st_icon_paint (ClutterActor *actor)
     }
 }
 
-static void
-st_icon_map (ClutterActor *actor)
-{
-  StIconPrivate *priv = ST_ICON (actor)->priv;
-
-  CLUTTER_ACTOR_CLASS (st_icon_parent_class)->map (actor);
-
-  if (priv->icon_texture)
-    clutter_actor_map (priv->icon_texture);
-}
-
-static void
-st_icon_unmap (ClutterActor *actor)
-{
-  StIconPrivate *priv = ST_ICON (actor)->priv;
-
-  CLUTTER_ACTOR_CLASS (st_icon_parent_class)->unmap (actor);
-
-  if (priv->icon_texture)
-    clutter_actor_unmap (priv->icon_texture);
-}
-
 static void
 st_icon_style_changed (StWidget *widget)
 {
@@ -292,7 +284,14 @@ st_icon_style_changed (StWidget *widget)
   StThemeNode *theme_node = st_widget_get_theme_node (widget);
   StIconPrivate *priv = self->priv;
 
-  priv->theme_icon_size = st_theme_node_get_length (theme_node, "icon-size");
+  if (priv->shadow_spec)
+    {
+      st_shadow_unref (priv->shadow_spec);
+      priv->shadow_spec = NULL;
+    }
+  priv->shadow_spec = st_theme_node_get_shadow (theme_node, "icon-shadow");
+
+  priv->theme_icon_size = (int)(0.5 + st_theme_node_get_length (theme_node, "icon-size"));
   st_icon_update_icon_size (self);
   st_icon_update (self);
 }
@@ -316,8 +315,6 @@ st_icon_class_init (StIconClass *klass)
   actor_class->get_preferred_width = st_icon_get_preferred_width;
   actor_class->allocate = st_icon_allocate;
   actor_class->paint = st_icon_paint;
-  actor_class->map = st_icon_map;
-  actor_class->unmap = st_icon_unmap;
 
   widget_class->style_changed = st_icon_style_changed;
 
@@ -355,12 +352,11 @@ st_icon_init (StIcon *self)
 {
   self->priv = ST_ICON_GET_PRIVATE (self);
 
-  self->priv->gicon = NULL;
   self->priv->icon_size = DEFAULT_ICON_SIZE;
   self->priv->prop_icon_size = -1;
   self->priv->icon_type = DEFAULT_ICON_TYPE;
 
-  self->priv->icon_texture = COGL_INVALID_HANDLE;
+  self->priv->shadow_material = COGL_INVALID_HANDLE;
   self->priv->shadow_width = -1;
   self->priv->shadow_height = -1;
 }
@@ -369,8 +365,6 @@ static void
 st_icon_update_shadow_material (StIcon *icon)
 {
   StIconPrivate *priv = icon->priv;
-  StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (icon));
-  StShadow *shadow_spec = st_theme_node_get_shadow (theme_node);
 
   if (priv->shadow_material)
     {
@@ -378,14 +372,15 @@ st_icon_update_shadow_material (StIcon *icon)
       priv->shadow_material = COGL_INVALID_HANDLE;
     }
 
-  if (shadow_spec)
+  if (priv->shadow_spec)
    {
      CoglHandle material;
      gint width, height;
 
      clutter_texture_get_base_size (CLUTTER_TEXTURE (priv->icon_texture),
                                     &width, &height);
-     material = _st_create_shadow_material_from_actor (shadow_spec,
+
+     material = _st_create_shadow_material_from_actor (priv->shadow_spec,
                                                        priv->icon_texture);
      priv->shadow_material = material;
      priv->shadow_width = width;
@@ -401,20 +396,62 @@ on_pixbuf_changed (ClutterTexture *texture,
 }
 
 static void
-st_icon_update (StIcon *icon)
+st_icon_finish_update (StIcon *icon)
 {
   StIconPrivate *priv = icon->priv;
-  StThemeNode *theme_node;
-  StTextureCache *cache;
 
-  /* Get rid of the old one */
   if (priv->icon_texture)
     {
       clutter_actor_destroy (priv->icon_texture);
       priv->icon_texture = NULL;
     }
 
-  /* Try to lookup the new one */
+  if (priv->pending_texture)
+    {
+      priv->icon_texture = priv->pending_texture;
+      priv->pending_texture = NULL;
+      clutter_actor_set_parent (priv->icon_texture, CLUTTER_ACTOR (icon));
+
+      /* Remove the temporary ref we added */
+      g_object_unref (priv->icon_texture);
+
+      st_icon_update_shadow_material (icon);
+
+      /* "pixbuf-change" is actually a misnomer for "texture-changed" */
+      g_signal_connect (priv->icon_texture, "pixbuf-change",
+                        G_CALLBACK (on_pixbuf_changed), icon);
+    }
+}
+
+static void
+opacity_changed_cb (GObject *object,
+                    GParamSpec *pspec,
+                    gpointer user_data)
+{
+  StIcon *icon = user_data;
+  StIconPrivate *priv = icon->priv;
+
+  g_signal_handler_disconnect (priv->pending_texture, priv->opacity_handler_id);
+  priv->opacity_handler_id = 0;
+
+  st_icon_finish_update (icon);
+}
+
+static void
+st_icon_update (StIcon *icon)
+{
+  StIconPrivate *priv = icon->priv;
+  StThemeNode *theme_node;
+  StTextureCache *cache;
+
+  if (priv->pending_texture)
+    {
+      clutter_actor_destroy (priv->pending_texture);
+      g_object_unref (priv->pending_texture);
+      priv->pending_texture = NULL;
+      priv->opacity_handler_id = 0;
+    }
+
   theme_node = st_widget_peek_theme_node (ST_WIDGET (icon));
   if (theme_node == NULL)
     return;
@@ -422,29 +459,41 @@ st_icon_update (StIcon *icon)
   cache = st_texture_cache_get_default ();
   if (priv->gicon)
     {
-      priv->icon_texture = st_texture_cache_load_gicon (cache,
-                                                        (priv->icon_type != ST_ICON_APPLICATION &&
-                                                         priv->icon_type != ST_ICON_DOCUMENT) ?
-                                                        theme_node : NULL,
-                                                        priv->gicon,
-                                                        priv->icon_size);
+      priv->pending_texture = st_texture_cache_load_gicon (cache,
+                                                           (priv->icon_type != ST_ICON_APPLICATION &&
+                                                            priv->icon_type != ST_ICON_DOCUMENT) ?
+                                                           theme_node : NULL,
+                                                           priv->gicon,
+                                                           priv->icon_size);
     }
  else if (priv->icon_name)
     {
-      priv->icon_texture = st_texture_cache_load_icon_name (cache,
-                                                            theme_node,
-                                                            priv->icon_name,
-                                                            priv->icon_type,
-                                                            priv->icon_size);
+      priv->pending_texture = st_texture_cache_load_icon_name (cache,
+                                                               theme_node,
+                                                               priv->icon_name,
+                                                               priv->icon_type,
+                                                               priv->icon_size);
     }
-  if (priv->icon_texture)
-    {
-      st_icon_update_shadow_material (icon);
-      clutter_actor_set_parent (priv->icon_texture, CLUTTER_ACTOR (icon));
 
-      /* "pixbuf-change" is actually a misnomer for "texture-changed" */
-      g_signal_connect (priv->icon_texture, "pixbuf-change",
-                        G_CALLBACK (on_pixbuf_changed), icon);
+  if (priv->pending_texture)
+    {
+      g_object_ref_sink (priv->pending_texture);
+
+      if (clutter_actor_get_opacity (priv->pending_texture) != 0 || priv->icon_texture == NULL)
+        {
+          /* This icon is ready for showing, or nothing else is already showing */
+          st_icon_finish_update (icon);
+        }
+      else
+        {
+          /* Will be shown when fully loaded */
+          priv->opacity_handler_id = g_signal_connect (priv->pending_texture, "notify::opacity", G_CALLBACK (opacity_changed_cb), icon);
+        }
+    }
+  else if (priv->icon_texture)
+    {
+      clutter_actor_destroy (priv->icon_texture);
+      priv->icon_texture = NULL;
     }
 }
 
